module TeachingPrintables
  # Creates quadrille rulings on paper.  Delegates Prawn methods to Prawn::Document.
  class TPDocument
    # Make quadrille rules on pdf page(s)
    #
    include Prawn::View
    
    DOC_OPTIONS_DEFAULT = {
      :page_layout => :portrait,
      :margin => 0
    }
    
    def initialize(args={})
      args = DOC_OPTIONS_DEFAULT.merge(args).merge({skip_page_creation: true})
      @document = Prawn::Document.new(args)
    end
  
    def page_width
      if page_layout == :portrait
        width = PDF::Core::PageGeometry::SIZES[page_size][0]
      else
        width = PDF::Core::PageGeometry::SIZES[page_size][1]
      end
    end
    
    def page_height
      if page_layout == :portrait
        height = PDF::Core::PageGeometry::SIZES[page_size][1]
      else
        height = PDF::Core::PageGeometry::SIZES[page_size][0]
      end
    end
    
    def method_missing(m, *args)
      method = m.to_s
      if document.class.public_method_defined?(method)
        document.send(method,*args)
      elsif method.start_with?("page_")
        page_method = method[5..-1]
        if state.page.class.public_method_defined?(page_method)
          state.page.send(page_method,*args)
        else 
          raise NoMethodError.new("Can't find method",page_method)
        end
      else
        super
      end
    end

  end
end

    
 